"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _globby = require("globby");

var _globby2 = _interopRequireDefault(_globby);

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

var _shelljs = require("shelljs");

var _shelljs2 = _interopRequireDefault(_shelljs);

var _markdownToc = require("markdown-toc");

var _markdownToc2 = _interopRequireDefault(_markdownToc);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const extractComments = str => {
  const lines = str.split("\n");

  const commentLines = lines.reduce(([lines, inComment], line) => {
    if (inComment && line.trim() === "* @private") return [lines, false];

    if (inComment && line.trim() === "*/") return [[...lines, "", ""], false];

    if (!inComment && line.trim() === "/**") return [lines, true];

    if (inComment) return [[...lines, line.trim().slice(2)], true];

    return [lines, false];
  }, [[], false])[0];

  return commentLines.join("\n");
};

const read = relativePath => {
  const absolutePath = _path2.default.resolve(process.cwd(), relativePath);
  const str = _shelljs2.default.cat(absolutePath);
  return str;
};

const makePaths = async globs => {
  const pathSets = await Promise.all(globs.map(_globby2.default));
  return _ramda2.default.flatten(pathSets.map(s => s.sort()));
};

const generateDocs = async ({ paths: globs, output }) => {
  const paths = await makePaths(globs);
  const comments = _ramda2.default.pipe(_ramda2.default.always(paths), _ramda2.default.map(_ramda2.default.ifElse(_ramda2.default.test(/\.md$/), read, _ramda2.default.compose(extractComments, read))), _ramda2.default.join("\n\n"), _markdownToc2.default.insert)();
  _shelljs2.default.echo(comments).to(_path2.default.resolve(process.cwd(), output));

  return true;
};

exports.default = generateDocs;